@@ -14,6 +14,8 @@ from pkg_resources import resource_string
 from typing import Tuple
 from urllib.parse import urlparse
 
+NO_ARG_VALUE = "NO_ARG_VALUE"
+
 
 def get_header(config_reader: ConfigReaderAndWriter, user: str, password: str, request_address: str) -> Tuple[dict, str]:
     config_all = config_reader.get_config()
@@ -36,9 +38,11 @@ def get_header(config_reader: ConfigReaderAndWriter, user: str, password: str, r
             return headers, request_address
         else:
             raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidCredentials"])
-    elif user is not None:
+    elif user is not None and password is not None:
         return get_basic_auth_header(user, password, request_address)
-    elif os.environ.get('GEOSPOCK_USER') is not None:
+    elif user is not None:
+        raise CLIError(ConfigReaderAndWriter.MESSAGES["userButNoPassword"])
+    elif os.environ.get('GEOSPOCK_USER') is not None and os.environ.get('GEOSPOCK_PASSWORD') is not None:
         return get_basic_auth_header(os.environ.get('GEOSPOCK_USER'),
                                      os.environ.get('GEOSPOCK_PASSWORD'),
                                      request_address)
@@ -47,7 +51,7 @@ def get_header(config_reader: ConfigReaderAndWriter, user: str, password: str, r
                                      config_reader.decode(config_all[config_reader.profile]["p"]),
                                      request_address)
     else:
-        raise CLIError(ConfigReaderAndWriter.MESSAGES["invalidCredentials"])
+        raise CLIError(ConfigReaderAndWriter.MESSAGES["insufficientLoginDetails"])
 
 
 def get_basic_auth_header(username: str, password: str, request_address: str) -> Tuple[dict, str]:
@@ -132,12 +136,8 @@ def run(ctx, command, profile, debug, no_browser, user, password, request_addres
         elif command == "profile-list":
             profile_list(config_reader)
 
-        # We want the number of additional arguments to be even (i.e. number % 2 == 0) so that every {--argumentName}
-        # corresponds to an {argumentValue}
-        elif len(ctx.args) % 2 != 0:
-            click.echo(ConfigReaderAndWriter.MESSAGES["argNumberWrong"])
         else:
-            args = {ctx.args[i][2:]: ctx.args[i + 1] for i in range(0, len(ctx.args), 2)}
+            args = get_arg_list(ctx.args)
             if command == "init":
                 # Saving info to config.json
                 try:
@@ -172,6 +172,22 @@ def run(ctx, command, profile, debug, no_browser, user, password, request_addres
         exit(e.exit_code)
 
 
+def get_arg_list(args):
+    i = 0
+    new_args = dict()
+    while i < len(args):
+        if args[i][:2] == "--":
+            if i + 1 >= len(args) or args[i + 1][:2] == "--":
+                new_args[args[i][2:]] = NO_ARG_VALUE
+                i += 1
+            else:
+                new_args[args[i][2:]] = args[i+1]
+                i += 2
+        else:
+            raise CLIError("Expecting argument name starting with `--`, found {}".format(args[i]))
+    return new_args
+
+
 def load_file(address):
     if address.startswith("s3://") or address.startswith("S3://"):
         parsed_url = urlparse(address)
@@ -226,11 +242,22 @@ def process_parameters(parameters: dict, args: dict, command: str) -> Tuple[dict
     for parameter in parameters:
         if (parameter["name"] in args and parameter["type"] in ["String", "Int", "ID", "Boolean"] and
                 ("fromFile" not in parameter or not parameter["fromFile"])):
-            new_args[parameter["internalName"]] = args[parameter["name"]]
+            if args[parameter["name"]] == NO_ARG_VALUE:
+                click.echo("Required argument --{0} not provided with a value for command {1}".format(parameter["name"],
+                                                                                                      command))
+                arg_missing = True
+            else:
+                new_args[parameter["internalName"]] = args[parameter["name"]]
             if parameter["type"] == "Int":
                 check_if_valid_int(parameter["name"], args[parameter["name"]])
         elif parameter["name"] in args and parameter["type"] == "enum":
-            if args[parameter["name"]] in parameter["enumValues"]:
+            if args[parameter["name"]] == NO_ARG_VALUE and "TRUE" in parameter["enumValues"]:
+                new_args[parameter["internalName"]] = "TRUE"
+            elif args[parameter["name"]] == NO_ARG_VALUE:
+                arg_missing = True
+                click.echo("Value not provided for argument --{0}. Allowed values: {1}".format(parameter["name"],
+                                                                                               parameter["enumValues"]))
+            elif args[parameter["name"]] in parameter["enumValues"]:
                 new_args[parameter["internalName"]] = args[parameter["name"]]
             else:
                 arg_missing = True
@@ -239,8 +266,12 @@ def process_parameters(parameters: dict, args: dict, command: str) -> Tuple[dict
                                                                                                   parameter["enumValues"]))
         elif parameter["name"] in args \
                 and ("fromFile" in parameter and parameter["fromFile"]):
-            file_contents = load_file(args[parameter["name"]])
-            new_args[parameter["internalName"]] = file_contents
+            if args[parameter["name"]] == NO_ARG_VALUE:
+                arg_missing = True
+                click.echo("No file location provided for argument --{0}.".format(parameter["name"]))
+            else:
+                file_contents = load_file(args[parameter["name"]])
+                new_args[parameter["internalName"]] = file_contents
         elif "default" in parameter:
             new_args[parameter["internalName"]] = parameter["default"]
         elif ("subParams" in parameter and len(parameter["subParams"]) == 0) and parameter["required"]:
